

#!/usr/bin/python2
# Copyright (c) Microsoft. All rights reserved.

# This is the utility to create and modify mssql.conf which contains
# the configurable settings to be used by SQL Server on Linux

# Note: This script targets Python 2

import argparse
from argparse import SUPPRESS
import sys
import os.path
import getpass
import subprocess
import mssqlconfhelper
import mssqlsettings
import mssqlsettingsmanager
from ConfigParser import ConfigParser

def handleWelcome():
    """Handle welcome command
    """

    if args.welcometype == "engine":
        exit(mssqlconfhelper.printEngineWelcomeMessage())
    elif args.welcometype == "agent":
        exit(mssqlconfhelper.printAgentWelcomeMessage())
    elif args.welcometype == "fts":
        exit(mssqlconfhelper.printFTSWelcomeMessage())
    elif args.welcometype == "polybase":
        exit(mssqlconfhelper.printPolyBaseWelcomeMessage())
    elif args.welcometype == "polybase-hadoop":
        exit(mssqlconfhelper.printPolyBaseHadoopWelcomeMessage())
    elif args.welcometype == "machinelearning":
        exit(mssqlconfhelper.printMachineLearningServicesWelcomeMessage())
    else:
        exit(mssqlconfhelper.errorExitCode)

def handleList():
    """Handle list command
    """

    mssqlconfhelper.listSupportedSettings(mssqlsettingsmanager.supportedSettingsList)

def handleSetup():
    """Handle setup command
    """

    if not mssqlconfhelper.checkSudo():
        mssqlconfhelper.printError(_("Setup must be run as the superuser. This can be done using 'sudo mssql-conf setup'."))
        exit(mssqlconfhelper.errorExitCode)

    eulaAccepted = False
    eulaMlAccepted = False

    if args.setup_option != None:
        if args.setup_option == "accept-eula":
            exit(mssqlconfhelper.setupSqlServer(True, noprompt=args.noprompt))
        elif args.setup_option == "accept-eula-ml":
            if mssqlconfhelper.isMlServicesInstalled():
                exit(mssqlconfhelper.setupSqlServerMlServices(True))
            else:
                print(_("Machine Learning services has not been installed."))
        else:
            print(_("The setup option of %s is not supported.") % (args.setup_option))
            print(_("The supported setup options are accept-eula and accept-eula-ml."))
            exit(mssqlconfhelper.errorExitCode)
    else:
        if "ACCEPT_EULA" in os.environ:
            eulaAccepted = True

        if "ACCEPT_EULA_ML" in os.environ:
            eulaMlAccepted = True

        mssqlconfhelper.setupSqlServer(eulaAccepted, noprompt=args.noprompt)

        if mssqlconfhelper.isMlServicesInstalled():
            mssqlconfhelper.setupSqlServerMlServices(eulaMlAccepted)

def handleValidate():
    """Handle validate command
    """

    if not os.path.exists(mssqlconfhelper.configurationFilePath):
        exit(mssqlconfhelper.successExitCode)

    # We set allow_no_value to be True so that validate can actually parse any syntax issues as well
    config = ConfigParser(allow_no_value=True)
    mssqlconfhelper.readConfigFromFile(config, mssqlconfhelper.configurationFilePath)

    if mssqlsettingsmanager.validateConfig(config):
        print(_("Validation successful."))
        exit(mssqlconfhelper.successExitCode)

    exit(mssqlconfhelper.errorExitCode)

def handleSet():
    """Handle set command
    """

    config = ConfigParser()
    mssqlconfhelper.readConfigFromFile(config, mssqlconfhelper.configurationFilePath)
    settingChosen = mssqlsettingsmanager.findSetting(args.section_name, args.setting_name)

    if settingChosen is None:
        mssqlconfhelper.printErrorUnsupportedSetting(args.section_name, args.setting_name)

    if settingChosen.sectionOnly:
        if mssqlsettingsmanager.setSectionKeyVal(config, settingChosen, args.setting_name, args.setting_value):
            mssqlconfhelper.writeConfigToFile(config, mssqlconfhelper.configurationFilePath)
    elif mssqlsettingsmanager.setSetting(config, settingChosen, args.setting_value):
            mssqlconfhelper.writeConfigToFile(config, mssqlconfhelper.configurationFilePath)
            if settingChosen.section == mssqlsettings.SectionForSetting.extensibility:
                mssqlconfhelper.printLaunchpadRestartRequiredMessage()

def handleUnset():
    """Handle unset command
    """

    settingToRemove = mssqlsettingsmanager.findSetting(args.section_name, args.setting_name)

    if settingToRemove is None:
        mssqlconfhelper.printErrorUnsupportedSetting(args.section_name, args.setting_name)

    config = ConfigParser()
    mssqlconfhelper.readConfigFromFile(config, mssqlconfhelper.configurationFilePath)

    if settingToRemove.sectionOnly:
        if mssqlsettingsmanager.unsetSectionKeyVal(config, settingToRemove, args.setting_name):
            mssqlconfhelper.writeConfigToFile(config, mssqlconfhelper.configurationFilePath)
    elif mssqlsettingsmanager.unsetSetting(config, settingToRemove):
            if settingToRemove.restart_required:
                mssqlconfhelper.printRestartRequiredMessage()
            mssqlconfhelper.writeConfigToFile(config, mssqlconfhelper.configurationFilePath)
            if settingToRemove.section == mssqlsettings.SectionForSetting.extensibility:
                mssqlconfhelper.printLaunchpadRestartRequiredMessage()

def handleSetSaPassword():
    """Handle set SA password command
    """

    ret = mssqlconfhelper.getSystemAdministratorPassword(noprompt=args.noprompt)

    if (ret == mssqlconfhelper.errorExitCode or ret == None):
        exit(mssqlconfhelper.errorExitCode)

    ret = mssqlconfhelper.configureSqlservrWithArguments("--setup --reset-sa-password", MSSQL_SA_PASSWORD=ret)

    if(ret == 0):
        print(_("The system administrator password has been changed."))
        mssqlconfhelper.printStartSqlServerMessage()
        exit(mssqlconfhelper.successExitCode)
    else:
        print(_("Unable to set the system administrator password. Please consult the ERRORLOG"))
        print(_("in %s for more information.") % (mssqlconfhelper.getErrorLogFile()))
        exit(ret)

def handleTraceflag():
    """Handle traceflag command
    """

    config = ConfigParser()
    mssqlconfhelper.readConfigFromFile(config, mssqlconfhelper.configurationFilePath)

    traceflagsList = args.traceflags_list
    if (args.tf_choice == 'on'):
        for traceflag in traceflagsList:
            mssqlsettingsmanager.addTraceFlag(config, traceflag)
    elif (args.tf_choice == 'off'):
        for traceflag in traceflagsList:
            mssqlsettingsmanager.removeTraceFlag(config, traceflag)

    mssqlconfhelper.writeConfigToFile(config, mssqlconfhelper.configurationFilePath)

def handleSetCollation():
    """Handle set collation command
    """

    collation = raw_input('Enter the collation: ')

    if not mssqlconfhelper.validateCollation(collation):
        exit(mssqlconfhelper.errorExitCode)

    ret = mssqlconfhelper.configureSqlservrWithArguments("-q%s" % (collation))

    if(ret == 0):
        print(_("The server collation has been changed."))
        mssqlconfhelper.printStartSqlServerMessage()
        exit(mssqlconfhelper.successExitCode)
    else:
        exit(mssqlconfhelper.errorExitCode)

def handleSetEdition():
    """Handle the set-edition command
    """

    if mssqlconfhelper.checkRunningInstance():
        exit(mssqlconfhelper.errorExitCode)

    pid = mssqlconfhelper.getPid(args.noprompt)

    if pid is None:
        exit(mssqlconfhelper.errorExitCode)

    ret = mssqlconfhelper.configureSqlservrWithArguments("--setup", MSSQL_PID=pid)

    if ret == mssqlconfhelper.successExitCode:
        mssqlconfhelper.printStartSqlServerMessage()
    else:
        exit(ret)


def handleGet():
    """
    Handle the get command which gets the setting values of the entire section specified or of a particular setting
    """
    settingsDictionary = mssqlconfhelper.getSettings(mssqlconfhelper.configurationFilePath,
                                                     args.section_name, args.setting_name)
    if not len(settingsDictionary):
        print (_("No setting for the given option found in '%s'.") % (mssqlconfhelper.configurationFilePath))

    for setting_name, setting_value in settingsDictionary.items():
        print("%s : %s" % (setting_name, setting_value))


def handleValidateADConfig():
    """
    Handle the check-ad-config command
    """

    config = mssqlconfhelper.adconfig(args.keytab, args.realm)
    config.validate()

def handleSetupADKeytab():
    """
    Handle the setup-ad-keytab command
    """
    mssqlconfhelper.setupADKeytab(args.keytab, args.user)

# Command dispatch table
#
COMMAND_TABLE = {
    "welcome": handleWelcome,
    "setup": handleSetup,
    "validate": handleValidate,
    "list": handleList,
    "set": handleSet,
    "unset": handleUnset,
    "traceflag": handleTraceflag,
    "set-sa-password": handleSetSaPassword,
    "set-collation": handleSetCollation,
    "set-edition": handleSetEdition,
    "get": handleGet,
    "validate-ad-config": handleValidateADConfig,
    "setup-ad-keytab": handleSetupADKeytab
}

def processCommands():
    """Process commands
    """

    COMMAND_TABLE[args.which]()
    exit(mssqlconfhelper.successExitCode)

def initializeCommandLine():
    """Initialize command line parsing
    """

    global args

    parser = argparse.ArgumentParser(prog='mssql-conf')

    parser.add_argument("-n", "--noprompt", action="store_true", help=_("Does not prompt the user and uses environment variables or defaults."))

    sp = parser.add_subparsers(metavar='')

    sp_setup = sp.add_parser('setup', help=_('Initialize and setup Microsoft SQL Server'))
    sp_setup.add_argument("setup_option", metavar="setup_option", help=_("Specify accept-eula to accept the End-User License Agreement"), nargs='?')
    sp_setup.set_defaults(which='setup')

    sp_set = sp.add_parser('set', help=_('Set the value of a setting'))
    sp_set.add_argument('section_name', metavar='SECTION', help=_('The section of the setting'), action='store')
    sp_set.add_argument('setting_name', metavar='SETTING', help=_('The setting name to set'), action='store', nargs='?')
    sp_set.add_argument('setting_value', metavar='VALUE', help=_('Value for the setting'), action='store')
    sp_set.set_defaults(which='set')

    sp_unset = sp.add_parser('unset', help=_('Unset the value of a setting'))
    sp_unset.add_argument('section_name', metavar='SECTION', help=_('The section of the setting'), action='store')
    sp_unset.add_argument('setting_name', metavar='SETTING', help=_('The setting name to unset'), action='store',
                          nargs='?')
    sp_unset.set_defaults(which='unset')

    sp_list = sp.add_parser('list', help=_('List the supported settings'))
    sp_list.set_defaults(which='list')

    sp_get = sp.add_parser('get', help=_('Gets the value of all settings in a section or of an individual setting'))
    sp_get.add_argument('section_name', metavar='SECTION', help=_('The section of the setting'), action='store')
    sp_get.add_argument('setting_name', metavar='SETTING', help=_('Optional: The setting whose value to get'), action='store',
                        nargs='?')
    sp_get.set_defaults(which='get')

    sp_traceflag = sp.add_parser('traceflag', help=_('Enable/disable one or more traceflags'))
    sp_traceflag.add_argument('traceflags_list', metavar='TRACEFLAGS', type=int, nargs='+',
                                     help=_('List of traceflags separated by whitespace.'))
    sp_traceflag.add_argument('tf_choice', metavar='on/off',choices=['on', 'off'], help=_('Turn the traceflags on or off.'))
    sp_traceflag.set_defaults(which='traceflag')

    sp_sa_password = sp.add_parser('set-sa-password',
                                   help=_('Set the system administrator (SA) password'))
    sp_sa_password.set_defaults(which='set-sa-password')

    sp_collation = sp.add_parser('set-collation',
                                help=_('Set the collation of system databases'))
    sp_collation.set_defaults(which='set-collation')

    sp_validate = sp.add_parser('validate',
                                help=_('Validate the configuration file'))
    sp_validate.set_defaults(which='validate')

    sp_set_edition = sp.add_parser('set-edition',
                                   help= _("Set the edition of the SQL Server instance"))
    sp_set_edition.set_defaults(which='set-edition')

    sp_welcome = sp.add_parser('welcome', usage=SUPPRESS)
    sp_welcome.add_argument("welcometype", metavar="welcometype", nargs='?')
    sp_welcome.set_defaults(which='welcome')

    sp_validateADConfig = sp.add_parser('validate-ad-config', help=_('Validate configuration for Active Directory Authentication'))
    sp_validateADConfig.set_defaults(which='validate-ad-config')
    sp_validateADConfig.add_argument('keytab', metavar="keytab", help=_("Path to keytab which SQL Server will use"), action="store", nargs="?", default="")
    sp_validateADConfig.add_argument('realm', metavar="realm", help=_("Realm which the host is joined to. Defaults to the default_realm value in /etc/krb5.conf"), action="store", nargs='?', default="")

    sp_setupADKeytab = sp.add_parser('setup-ad-keytab', help=_('Create a keytab for SQL Server to use to authenticate AD users'))
    sp_setupADKeytab.add_argument('keytab', metavar="keytab", help=_('Path to the keytab that will be created'), action="store")
    sp_setupADKeytab.add_argument('user', metavar="user", help=_('AD Managed Service Account (MSA) which owns the SPNs'), action="store", nargs="?", default="")
    sp_setupADKeytab.set_defaults(which='setup-ad-keytab')

    if (len(sys.argv) == 1):
        parser.print_help()
        exit(mssqlconfhelper.errorExitCode)

    args = parser.parse_args()

def main():
    """Program main function
    """

    mssqlconfhelper.initialize()
    mssqlsettingsmanager.initialize()

    if not mssqlconfhelper.checkSudoOrMssql():
        print(_("This program must be run as superuser or as a user with membership in the mssql"))
        print(_("group."))
        exit(mssqlconfhelper.errorExitCode)

    initializeCommandLine()
    processCommands()
    exit(mssqlconfhelper.successExitCode)

if __name__ == "__main__":
    main(